### Tutorial on how to use scripts to reproduce our results

#### Building a Docker image for the analysis
In order to build a Docker image your user name should be in the docker group. So, when you try to run the following command you may get *a permission error*, which means you need to be added to the docker by a sysadmin.

`docker build -t  ejprd/cakut_mixomics:0.1 -f Dockerfile_mixomics .`  

If everything goes fine then a docker image will be created, which you can check using the following command on the command line:  
`docker images|grep cakut_mixomics` 

If the previous command returns nothing, which is very unlikely, something must have gone wrong.

#### Running  

##### Single-omics analysis  
Run the command as following, but all input files must be in the same directory with the `separate_single_omics_analysis.R` script:  
`mkdir singleomics_run`  
`docker run --rm -v $(pwd)/:/data/ ejprd/cakut_mixomics:0.1 bash -c "cd /data/ && R -f separate_single_omics_analysis.R --args /data/mirnome_training_data_common_format.tsv /data/mirnome_validation_data_common_format.tsv ./singleomics_run randomforest,pca,spca,plsda,splsda`

Here is the general format:  
`docker run --rm -v $(pwd)/:/data/ ejprd/cakut_mixomics:0.1 bash -c "cd /data/ && R -f separate_single_omics_analysis.R --args omics_training_file omics_validation_file output_folder methods`

The methods option may contain any combinations of the following option(s): *randomforest,pca,spca,plsda,splsda*. However, in the case of sparse PCA and sparse PLS-DA, default of 2 components and for each component 100 features would be used. The methods should contain comma-delimited method names and there can be space between method names.

##### Multi-omics analysis  
Run the command as following, but all input files must be in the same directory with the `separate_single_omics_analysis.R` script:  
`mkdir multiomics_run`  
`docker run --rm -v $(pwd)/:/data/ ejprd/cakut_mixomics:0.1 bash -c "cd /data/ && R -f multi_omics_analysis.R --args /data/mirnome_training_data_common_format.tsv,/data/peptidome_training_data_common_format.tsv /data/mirnome_validation_data_common_format.tsv,/data/peptidome_validation_data_common_format.tsv miRNome,peptidome multiomics_run diablo,multiomics_plsda`

Here is the general format:  
`docker run --rm -v $(pwd)/:/data/ ejprd/cakut_mixomics:0.1 bash -c "cd /data/ && R -f multi_omics_analysis.R --args omics_training_files omics_validation_files omics_names output_folder methods`

This script takes as arguments list of training files and validations files both are comma-separated. So, there should be at least 2 training files and 2 corresponding validation files. Please do not mix the order! The argument omics_names requires comma-separated list of omics data types, which are used in visualizations. Currently only DIABLO and multiomics plsda of the mixOmics package are implemented; thus, use diablo and/or multiomics_plsda only. The methods option may contain any combinations of the following option(s): *diablo,multiomics_plsda*. However, in the case of DIABLO, default of 2 components would be used. The methods should contain comma-delimited method names and there can be space between method names.


*Input files must be tab-delimited and all needs to be in the same folder as mentioned above.*    

### Notes:
System information
We successfully followed the documentation steps described for each analysis approach using the following configuration:  
Linux: CentOS Linux 7 (Core)  
Docker server/client version: 20.10.2 
R version in the Docker image is 4.0.2.

Note that we used `bash`, so certain commands may not work properly on another shells.


#### Reference
Check mixomics.org for more information about methods based on the mixOmics package and check for Random Forest algorithm from Leo Breiman.