# Single-omics analysis part, maybe make this one a separate R file

read_omics_file <- function(omics_file){
    ## Read the training file
    omics_df1 <- read.table(omics_file, sep="\t", header=F, row.names=1)
    # Remove first 2 rows and use it as a column name and separate column, respectively
    omics_df <- as.data.frame(t(omics_df1[3:nrow(omics_df1),]), stringsAsFactors = F); 
    omics_df <- as.data.frame(cbind(t(omics_df1[2,]),omics_df))
    rownames(omics_df) <- as.vector(unlist(omics_df1[1,]))
    # Since the 2nd row contains response/class/status inormation we want to use that as a response variable
    colnames(omics_df)[1] <- "response";
    return(omics_df);
}

diablo_analysis <- function(X_train, Y_train, X_valid, Y_valid, output_folder,keepX_diablo, diablo_components=2){
    print("Names of training data X in function diablo_analysis")
    print(names(X_train))

    pdf(paste(output_folder,"/multi_omics_diablo_results.pdf",sep=""));

    diablo_multiomics = block.splsda(X=X_train,Y=Y_train, keepX = keepX_diablo, ncomp = diablo_components);
    # Variable selection at 3 different levels: 1) features significant for two omics; 2) only for omics 1 ; or 3) only for omics 2.
    #variables selected and loadings values on component 1 for the two blocs
    important_var_for_all_omics = selectVar(diablo_multiomics, comp = 1, block = c(1,2)) 
    write.table(file=paste(output_folder, "/diablo_important_var_for_all_omics.tsv", sep=""), important_var_for_all_omics, sep="\t", row.names=F, col.names=F)

    # variables selected on component 1 for each block
    important_var_for_omics1 = selectVar(diablo_multiomics, comp = 1, block = c(1,2))[[1]] # Get important vars from the 1st block
    important_var_for_omics2 = selectVar(diablo_multiomics, comp = 1, block = c(1,2))[[2]] # Get important vars from the 2nd block
    write.table(file=paste(output_folder, "/diablo_important_var_for_",names(X_train)[1], ".tsv", sep=""), important_var_for_omics1, sep="\t", row.names=F, col.names=T)
    write.table(file=paste(output_folder, "/diablo_important_var_for_",names(X_train)[2], ".tsv", sep=""), important_var_for_omics2, sep="\t", row.names=F, col.names=T)

    # important_var_for_all_omics = selectVar(diablo_multiomics, comp = 1, block = c(1,2)) 
    # important_var_for_all_omics = selectVar(diablo_multiomics, comp = 1, block = c(1,2)) 

    plotIndiv(diablo_multiomics, legend = TRUE, title="DIABLO Results on Training Data", legend.title="Class",comp=c(1,2), cex=c(2.5,2.5));
    plotVar(diablo_multiomics,cutoff=0.8,var.names=T, title="DIABLO on Training Data: Features with Correlations > 80%",cex=c(2.5,2.5))
    plotLoadings(diablo_multiomics,group=diablo_multiomics$response, legend = TRUE, method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1, contrib = "max", cex=2.5); #cex=0.8, title="DIABLO Loadings on Training Data\nComponent 1", legend.title="Class", 
    plotLoadings(diablo_multiomics,group=diablo_multiomics$response, legend = TRUE, method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2, contrib = "max", cex=2.5); #cex=0.8, title="DIABLO Loadings on Training Data\nComponent 2", legend.title="Class",
    network(diablo_multiomics, cutoff = 0.8, show.color.key = T, color.node=c("red", "green"))# legend = TRUE,  title="DIABLO Robust Pearson Correlation Network on Training Data\nComponent 1", comp=1);
    #network(diablo_multiomics, cutoff = 0.8, show.color.key = T, comp=2)#,legend = TRUE, title="DIABLO Robust Pearson Correlation Network on Training Data\nComponent 2", comp=2);
    plotDiablo(diablo_multiomics, ncomp = 1, legend = TRUE, title="DIABLO Pearson Correlation of Omics Data on Training Data\nComponent 1");
    mtext("DIABLO Pearson Correlation of Omics Data on Training Data\nComponent 1", side=1)
    plotDiablo(diablo_multiomics, ncomp = 2, legend = TRUE, title="DIABLO Pearson Correlation of Omics Data on Training Data\nComponent 2")
    mtext("DIABLO Pearson Correlation of Omics Data on Training Data\nComponent 2", side=1)
    circosPlot(diablo_multiomics, cutoff = 0.8, comp = 1:2);
    cimDiablo(diablo_multiomics, margin = c(4,18), size.legend = 1);

    auroc(diablo_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the DIABLO on Validation Data\nComponent 1 of ",names(X_valid)[1]), roc.comp=1, roc.block=1);
    auroc(diablo_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the DIABLO on Validation Data\nComponent 1 of ",names(X_valid)[2]), roc.comp=1, roc.block=2);
    auroc(diablo_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the DIABLO on Validation Data\nComponent 2 of ", names(X_valid)[1]), roc.comp=2, roc.block=1);
    auroc(diablo_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the DIABLO on Validation Data\nComponent 2 of ", names(X_valid)[2]), roc.comp=2, roc.block=2);
    dev.off();
    
    predicted_diablo_multiomics = predict(diablo_multiomics, newdata = X_valid);
    conf_diablo_multiomics = get.confusion_matrix(truth = Y_valid, predicted = predicted_diablo_multiomics$MajorityVote$centroids.dist[,2]);
    ber_error = get.BER(conf_diablo_multiomics);
    
    diablo_multiomics_results <- as.data.frame(lapply(predicted_diablo_multiomics$class$centroids.dist,function(x){return(x[,1])}));
    diablo_multiomics_results <- cbind(as.character(Y_valid),diablo_multiomics_results)
    colnames(diablo_multiomics_results)[1] <- c("actual");
    colnames(diablo_multiomics_results) <- gsub(pattern = "^",replacement = "validation_",x = colnames(diablo_multiomics_results));
    # X_valid and Y_valid have the same row names, but since Y_valid is a vector I have not assigned rownames to it.
    rownames(diablo_multiomics_results) <- rownames(X_valid[[1]]);
    # However, I need to revalidate that rownames of the predicted classes should have an identical sample names
    sample_names_in_predicted_results = as.data.frame(lapply(predicted_diablo_multiomics$class$centroids.dist,function(x){rownames(x)}))
    if(!all(ifelse(apply(sample_names_in_predicted_results,MARGIN=1,FUN=function(x){length(unique(x))})==1,TRUE,FALSE))){
        print("Somehow sample names of different omics data sets in the predicted data (based on the validation data) are NOT equal, this is WRONG. Below is the sample names table. Exiting!");
        print(sample_names_in_predicted_results);
        quit(save="no");
    }
    # If previous if condition passes succesfully then we should also compare it to sample names in the X_valid matrix
    if(!all.equal(row.names(X_valid[[1]]),sample_names_in_predicted_results[,1])){
        print("Somehow sample names of validation data do NOT match to the sample names in the predicted data (based on the validation data), this is WRONG. Exiting!");
        print(row.names(X_valid[[1]]))
        print(sample_names_in_predicted_results[,1]);
        quit(save="no");
    }
    
    
    write.table(file=paste(output_folder, "/diablo_multiomics_validation_actual_vs_predicted.tsv", sep=""), diablo_multiomics_results, sep="\t", row.names=T, col.names=NA)
    write.table(file=paste(output_folder,"/diablo_multiomics_validation_confusion_matrix.tsv",sep=""), conf_diablo_multiomics,col.names = NA,sep="\t")
    write.table(file=paste(output_folder,"/diablo_multiomics_BER_error.txt",sep=""), ber_error,col.names = NA,sep="\t")

}


# The following function multiomics_plsda_analysis is similar to the diablo_analysis function above. Major difference is that DIABLO does feature selection and block.plsda doesn't
multiomics_plsda_analysis <- function(X_train, Y_train, X_valid, Y_valid, output_folder, diablo_components=2){

    pdf(paste(output_folder,"/multi_omics_plsda_results.pdf",sep=""));

    plsda_multiomics = block.plsda(X=X_train,Y=Y_train, ncomp = diablo_components);

     # Variable selection at 3 different levels: 1) features significant for two omics; 2) only for omics 1 ; or 3) only for omics 2.
    #variables selected and loadings values on component 1 for the two blocs
    block_plsda_variates = plsda_multiomics$variates
    block_plsda_loadings = plsda_multiomics$Loadings
    block_plsda_names = plsda_multiomics$names
    print("block.plsda variates, loadings, names")
    print(block_plsda_variates)
    print("loadings")
    print(names(block_plsda_loadings))
    print("nzv")
    print(plsda_multiomics$nzv)
    print("Explained variance")
    print(plsda_multiomics$prop_expl_var)

    # important_var_for_all_omics = selectVar(plsda_multiomics, comp = 1, block = c(1,2))
    # print(important_var_for_all_omics)
    # write.table(file=paste(output_folder, "/block_plsda_variates.tsv", sep=""), block_plsda_variates, sep="\t", col.names=T, row.names=F)

    # variables selected on component 1 for each block
    important_var_for_omics1 = selectVar(plsda_multiomics, comp = 1, block = c(1,2))[[1]] # Get important vars from the 1st block
    important_var_for_omics2 = selectVar(plsda_multiomics, comp = 1, block = c(1,2))[[2]] # Get important vars from the 2nd block
    
    write.table(file=paste(output_folder, "/block_plsda_important_var_for_",names(X_train)[1], ".tsv", sep=""), important_var_for_omics1, sep="\t", row.names=F, col.names=T)
    write.table(file=paste(output_folder, "/block_plsda_important_var_for_",names(X_train)[2], ".tsv", sep=""), important_var_for_omics2, sep="\t", row.names=F, col.names=T)


    plotIndiv(plsda_multiomics, legend = TRUE, title="Multiomics PLS-DA Results on Training Data", legend.title="Class",comp=c(1,2), cex=c(2.5,2.5));
    plotVar(plsda_multiomics,cutoff=0.8,var.names=T, title="Multiomics PLS-DA on Training Data: Features with Correlations > 80%",cex=c(2.5,2.5))
    plotLoadings(plsda_multiomics,group=plsda_multiomics$response, legend = TRUE, method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1, contrib = "max", cex=2.5); #cex=0.8, title="Multiomics PLS-DA Loadings on Training Data\nComponent 1", legend.title="Class", 
    plotLoadings(plsda_multiomics,group=plsda_multiomics$response, legend = TRUE, method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2, contrib = "max", cex=2.5); #cex=0.8, title="Multiomics PLS-DA Loadings on Training Data\nComponent 2", legend.title="Class",
    network(plsda_multiomics, cutoff = 0.8, show.color.key = T, color.node=c("red", "green"))# legend = TRUE,  title="Multiomics PLS-DA Robust Pearson Correlation Network on Training Data\nComponent 1", comp=1);
    #network(plsda_multiomics, cutoff = 0.8, show.color.key = T, comp=2)#,legend = TRUE, title="Multiomics PLS-DA Robust Pearson Correlation Network on Training Data\nComponent 2", comp=2);
    plotDiablo(plsda_multiomics, ncomp = 1, legend = TRUE, title="Multiomics PLS-DA Pearson Correlation of Omics Data on Training Data\nComponent 1");
    mtext("Multiomics PLS-DA Pearson Correlation of Omics Data on Training Data\nComponent 1", side=1)
    plotDiablo(plsda_multiomics, ncomp = 2, legend = TRUE, title="Multiomics PLS-DA Pearson Correlation of Omics Data on Training Data\nComponent 2")
    mtext("Multiomics PLS-DA Pearson Correlation of Omics Data on Training Data\nComponent 2", side=1)
    # circosPlot(plsda_multiomics, cutoff = 0.8, comp = 1:2);
    # cimDiablo(plsda_multiomics, margin = c(4,18), size.legend = 1);

    auroc(plsda_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the Multiomics PLS-DA on Validation Data\nComponent 1 of ",names(X_valid)[1]), roc.comp=1, roc.block=1);
    auroc(plsda_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the Multiomics PLS-DA on Validation Data\nComponent 1 of ",names(X_valid)[2]), roc.comp=1, roc.block=2);
    auroc(plsda_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the Multiomics PLS-DA on Validation Data\nComponent 2 of ", names(X_valid)[1]), roc.comp=2, roc.block=1);
    auroc(plsda_multiomics, newdata=X_valid, outcome.test = Y_valid, title=paste("ROC Curve of the Multiomics PLS-DA on Validation Data\nComponent 2 of ", names(X_valid)[2]), roc.comp=2, roc.block=2);
    dev.off();
    
    predicted_plsda_multiomics = predict(plsda_multiomics, newdata = X_valid);
    conf_plsda_multiomics = get.confusion_matrix(truth = Y_valid, predicted = predicted_plsda_multiomics$MajorityVote$centroids.dist[,2]);
    ber_error = get.BER(conf_plsda_multiomics);
    
    plsda_multiomics_results <- as.data.frame(lapply(predicted_plsda_multiomics$class$centroids.dist,function(x){return(x[,1])}));
    plsda_multiomics_results <- cbind(as.character(Y_valid),plsda_multiomics_results)
    colnames(plsda_multiomics_results)[1] <- c("actual");
    colnames(plsda_multiomics_results) <- gsub(pattern = "^",replacement = "validation_",x = colnames(plsda_multiomics_results));
    # X_valid and Y_valid have the same row names, but since Y_valid is a vector I have not assigned rownames to it.
    rownames(plsda_multiomics_results) <- rownames(X_valid[[1]]);
    # However, I need to revalidate that rownames of the predicted classes should have an identical sample names
    sample_names_in_predicted_results = as.data.frame(lapply(predicted_plsda_multiomics$class$centroids.dist,function(x){rownames(x)}))
    
    
    if(!all(ifelse(apply(sample_names_in_predicted_results,MARGIN=1,FUN=function(x){length(unique(x))})==1,TRUE,FALSE))){
        print("Somehow sample names of different omics data sets in the predicted data (based on the validation data) are NOT equal, this is WRONG. Below is the sample names table. Exiting!");
        print(sample_names_in_predicted_results);
        quit(save="no");
    }
    # If previous if condition passes succesfully then we should also compare it to sample names in the X_valid matrix
    if(!all.equal(row.names(X_valid[[1]]),sample_names_in_predicted_results[,1])){
        print("Somehow sample names of validation data do NOT match to the sample names in the predicted data (based on the validation data), this is WRONG. Exiting!");
        print(row.names(X_valid[[1]]))
        print(sample_names_in_predicted_results[,1]);
        quit(save="no");
    }
    write.table(file=paste(output_folder, "/plsda_multiomics_validation_actual_vs_predicted.tsv", sep=""), plsda_multiomics_results, sep="\t", row.names=T, col.names=NA)
    write.table(file=paste(output_folder,"/plsda_multiomics_validation_confusion_matrix.tsv",sep=""), conf_plsda_multiomics,col.names = NA,sep="\t")
    write.table(file=paste(output_folder,"/plsda_multiomics_BER_error.txt",sep=""), ber_error,col.names = NA,sep="\t")

}

## Data file names
args <- commandArgs(trailingOnly=TRUE)
if(length(args)<5){
    cat("Usage:\n\tR --args omics_training_files omics_validation_files omics_names output_folder methods\n");
    cat("This script takes as arguments list of training files and validations files both are comma-separated\n");
    cat("So, there should be at least 2 training files and 2 corresponding validation files. Please do not mix the order!\n");
    cat("The argument omics_names requires comma-separated list of omics data types, which are used in visualization.\n");
    cat("Currently only DIABLO and multiomics plsda of the mixOmics package are implemented; thus, use diablo and/or multiomics_plsda only.\n");
    cat("E.g: R -f multi_omics_analysis.R --args peptidome_training.tsv,mirnome_training.tsv peptidome_validation.tsv,mirnome_validation.tsv peptidome,miRNome output_folder diablo,multiomics_plsda\n");
    q(save="no");
}
library(mixOmics)
library(ggplot2)
omics_training_files=unlist(strsplit(args[1],","));
omics_validation_files=unlist(strsplit(args[2],","));
omics_types = unlist(strsplit(args[3],","));
output_folder= args[4];
methods = unlist(strsplit(args[5],","));


# There should be at least two different omics files to do multi-omics data analysis
if(length(omics_training_files)!=length(omics_validation_files)){
    print("There must be equal number of training and validation files. Exiting!");
    quit(save="no");
}
if(length(omics_training_files) < 2){
    print("There must be at least 2 omics files to do a multi-omics analysis. Exiting!");
    quit(save="no");
}
if(length(omics_training_files)!=length(omics_types)){
    print("Number of omics types need to be equal to the omics training/validation files. Exiting!");
    quit(save="no");
}
omics_training_set=list()
omics_validation_set=list()
# Read training files into a list, where each list item is a data frame with the omics data.
omics_training_set <- lapply(omics_training_files,function(f){read_omics_file(f)});
names(omics_training_set) <- as.character(omics_types);

# Read validation files into a list, where each list item is a data frame with the omics data.
omics_validation_set <- lapply(omics_validation_files,function(f){read_omics_file(f)});
names(omics_validation_set) <- as.character(omics_types);

# Validate if the sample names match in omics files
omics_training_samples <- lapply(omics_training_set,rownames);
omics_training_common_samples <- Reduce(intersect,omics_training_samples);
print(paste("In total there are ", length(omics_training_common_samples), " common training samples"));
omics_validation_samples <- lapply(omics_validation_set,rownames)
omics_validation_common_samples <- Reduce(intersect,omics_validation_samples);
print(paste("In total there are ", length(omics_validation_common_samples), " common validation samples"));

# Prepare training blocks
X_train = lapply(omics_training_set,function(x){as.data.frame(x[omics_training_common_samples,-which(names(x) %in% c("response"))])})
X_train = lapply(X_train,function(x1){as.data.frame(lapply(x1,function(x2){as.double(as.character(x2))}))});
X_train = lapply(X_train, function(x){rownames(x) <- omics_training_common_samples;return(x)});
Y_train = omics_training_set[[1]][omics_training_common_samples,c("response")];


# Prepare validation blocks
X_valid = lapply(omics_validation_set,function(x){as.data.frame(x[omics_validation_common_samples,-which(names(x) %in% c("response"))])})
X_valid = lapply(X_valid,function(x1){as.data.frame(lapply(x1,function(x2){as.double(as.character(x2))}))});
X_valid = lapply(X_valid, function(x){rownames(x) <- omics_validation_common_samples;return(x)});
Y_valid = omics_validation_set[[1]][omics_validation_common_samples,c("response")];

# Feature selecion params for DIABLO
diablo_components=length(omics_types);
feature_count=50;
# So, next line means use feature_count features for each omics data type
keepX_diablo = lapply(omics_training_set,function(x){rep(min(feature_count,ncol(x)-1),length(omics_types))});
names(keepX_diablo) <- names(X_train);

# Call the relevant function per method
for(i in 1:length(methods)){
    switch(tolower(methods[i]),
           diablo=diablo_analysis(X_train, Y_train, X_valid, Y_valid, output_folder, keepX_diablo, diablo_components),
           multiomics_plsda=multiomics_plsda_analysis(X_train, Y_train, X_valid, Y_valid, output_folder, diablo_components)
    )
}

